@@ -3,38 +3,43 @@
 #' @name coords
 #' @param x (data.frame) A data.frame
 #' @param lat,lon (character) Latitude and longitude column to use. See Details.
-#' @param field (character) Name of filed in input data.frame x with country names
+#' @param field (character) Name of field in input data.frame x with country
+#' names
 #' @param country (character) A single country name
-#' @param drop (logical) Drop bad data points or not. Either way, we parse
-#' out bade data points as an attribute you can access. Default: \code{TRUE}
-#'
+#' @param which (character) one of "has_dec", "no_zeros", or "both" (default)
+#' @param drop (logical) Drop bad data points or not. Either way, we parse out
+#' bad data points as an attribute you can access. Default: `TRUE`
+#' @param ignore.na (logical) To consider NA values as a bad point or not.
+#' Default: `FALSE`
+#' @param coorduncertainityLimit (numeric) numeric threshold for the
+#' coordinateUncertainityInMeters variable. Default: 30000
 #' @return Returns a data.frame, with attributes
-#'
 #' @details
 #' Explanation of the functions:
 #'
-#' \itemize{
-#'  \item coord_impossible - Impossible coordinates
-#'  \item coord_incomplete - Incomplete coordinaes
-#'  \item coord_pol_centroids - Points at political centroids
-#'  \item coord_unlikely - Unlikely coordinates
-#'  \item coord_within - Check if points are within user input
-#'  political boundaries
-#' }
+#' - coord_impossible - Impossible coordinates
+#' - coord_incomplete - Incomplete coordinates
+#' - coord_imprecise - Imprecise coordinates
+#' - coord_pol_centroids - Points at political centroids
+#' - coord_unlikely - Unlikely coordinates
+#' - coord_within - Filter points within user input
+#' political boundaries
+#' - coord_uncertain - Uncertain occurrances of measured through
+#' coordinateUncertaintyInMeters default limit= 30000
 #'
 #' If either lat or lon (or both) given, we assign the given column name
-#' to be standardized names of "latitude", and "longitude". If not given, we attempt
-#' to guess what the lat and lon column names are and assign the same standardized
-#' names. Assigning the same standardized names makes downstream processing easier
-#' so that we're dealing with consistent column names. On returning the data, we
-#' return the original names.
+#' to be standardized names of "latitude", and "longitude". If not given, we
+#' attempt to guess what the lat and lon column names are and assign the
+#' same standardized names. Assigning the same standardized names makes
+#' downstream processing easier so that we're dealing with consistent column
+#' names. On returning the data, we return the original names.
 #'
-#' For \code{coord_within}, we use \code{countriesLow} dataset from the
+#' For `coord_within`, we use `countriesLow` dataset from the
 #' \pkg{rworldmap} package to get country borders.
 #'
 #' @section coord_pol_centroids:
 #' Right now, this function only deals with city centroids, using the
-#' \code{\link[maps]{world.cities}} dataset of more than 40,000 cities.
+#' [maps::world.cities] dataset of more than 40,000 cities.
 #' We'll work on adding country centroids, and perhaps others (e.g.,
 #' counties, states, provinces, parks, etc.).
 #'
@@ -54,6 +59,25 @@
 #' NROW(df_inc)
 #' attr(df_inc, "coord_incomplete")
 #'
+#'
+#' # Remove imprecise cases
+#' df <- sample_data_5
+#' NROW(df)
+#' ## remove records that don't have decimals at all
+#' df_imp <- dframe(df) %>% coord_imprecise(which = "has_dec")
+#' NROW(df_imp)
+#' attr(df_imp, "coord_imprecise")
+#' ## remove records that have all zeros
+#' df_imp <- dframe(df) %>% coord_imprecise(which = "no_zeros")
+#' NROW(df_imp)
+#' attr(df_imp, "coord_imprecise")
+#' ## remove both records that don't have decimals at all and those that
+#' ## have all zeros
+#' df_imp <- dframe(df) %>% coord_imprecise(which = "both")
+#' NROW(df_imp)
+#' attr(df_imp, "coord_imprecise")
+#'
+#'
 #' # Remove unlikely points
 #' NROW(df)
 #' df_unlikely <- dframe(df) %>% coord_unlikely()
@@ -63,22 +87,24 @@
 #' # Remove points not within correct political borders
 #' if (requireNamespace("rgbif", quietly = TRUE)) {
 #'    library("rgbif")
-#'    wkt <- 'POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))'
-#'    res <- rgbif::occ_data(geometry = wkt, limit=100)$data
+#'    wkt <- 'POLYGON((30.1 10.1,40 40,20 40,10 20,30.1 10.1))'
+#'    res <- rgbif::occ_data(geometry = wkt, limit=300)$data
 #' } else {
 #'    res <- sample_data_4
 #' }
 #'
 #' ## By specific country name
 #' NROW(res)
-#' df_within <- dframe(res) %>% coord_within(country = "Egypt")
+#' df_within <- dframe(res) %>% coord_within(country = "Israel")
 #' NROW(df_within)
 #' attr(df_within, "coord_within")
 #'
-#' ## By a field in your data - makes sure your points occur in one of those countries
+#' ## By a field in your data - makes sure your points occur in one
+#' ## of those countries
 #' NROW(res)
 #' df_within <- dframe(res) %>% coord_within(field = "country")
 #' NROW(df_within)
+#' head(df_within)
 #' attr(df_within, "coord_within")
 #'
 #' # Remove those very near political centroids
@@ -96,6 +122,22 @@
 #' df[1, "mylat"] <- 170
 #' dframe(df) %>% coord_impossible(lat = "mylat", lon = "mylon")
 #'
+#' df <- sample_data_6
+#'
+#' # Remove uncertain occurances
+#'
+#' NROW(df)
+#' df1<-df %>% coord_uncertain()
+#' NROW(df1)
+#' attr(df, "coord_uncertain")
+#'
+#' NROW(df)
+#' df2<-df %>% coord_uncertain(coorduncertainityLimit = 20000)
+#' NROW(df2)
+#'
+#' NROW(df)
+#' df3<-df %>% coord_uncertain(coorduncertainityLimit = 20000,ignore.na=TRUE)
+#' NROW(df3)
 
 #' @export
 #' @rdname coords
@@ -103,18 +145,80 @@ coord_incomplete <- function(x, lat = NULL, lon = NULL, drop = TRUE) {
   x <- do_coords(x, lat, lon)
   incomp <- x[!complete.cases(x$latitude, x$longitude), ]
   if (NROW(incomp) == 0) incomp <- NA
-  if (drop) x <- x[complete.cases(x$latitude, x$longitude), ]
+  if (drop) {
+    x <- add_atts(x[complete.cases(x$latitude, x$longitude), ], get_atts(x))
+  }
   row.names(incomp) <- NULL
   row.names(x) <- NULL
   structure(reassign(x), coord_incomplete = incomp)
 }
 
+get_atts <- function(x) {
+  attributes(x)[names(attributes(x)) %in%
+                  c('lat_var', 'lon_var', 'lat_var_orig', 'lon_var_orig')]
+}
+
+add_atts <- function(x, atts) {
+  for (i in seq_along(atts)) {
+    attr(x, names(atts)[i]) <- atts[[i]]
+  }
+  return(x)
+}
+
+#' @export
+#' @rdname coords
+coord_imprecise <- function(x, which = "both", lat = NULL, lon = NULL,
+  drop = TRUE) {
+
+  x <- do_coords(x, lat, lon)
+  switch(
+    which,
+    has_dec = {
+      incomp <- x[!grepl("[0-9]+\\.[0-9]+", x$longitude) | !grepl("[0-9]+\\.[0-9]+", x$latitude), ]
+    },
+    no_zeros = {
+      incomp <- x[grepl("[0-9]+\\.[0]+$", x$longitude) | grepl("[0-9]+\\.[0]+$", x$latitude), ]
+    },
+    both = {
+      incomp1 <- x[!grepl("[0-9]+\\.[0-9]+", x$longitude) | !grepl("[0-9]+\\.[0-9]+", x$latitude), ]
+      incomp2 <- x[grepl("[0-9]+\\.[0]+$", x$longitude) | grepl("[0-9]+\\.[0]+$", x$latitude), ]
+      incomp <- rbind(incomp1, incomp2)
+      incomp <- incomp[!duplicated(incomp), ]
+    }
+  )
+
+  if (NROW(incomp) == 0) incomp <- NA
+  if (drop) {
+    switch(
+      which,
+      has_dec = {
+        x <- x[grepl("[0-9]+\\.[0-9]+", x$longitude), ]
+        x <- x[grepl("[0-9]+\\.[0-9]+", x$latitude), ]
+      },
+      no_zeros = {
+        x <- x[!grepl("[0-9]+\\.[0]+$", x$longitude), ]
+        x <- x[!grepl("[0-9]+\\.[0]+$", x$latitude), ]
+      },
+      both = {
+        x <- x[grepl("[0-9]+\\.[0-9]+", x$longitude), ]
+        x <- x[grepl("[0-9]+\\.[0-9]+", x$latitude), ]
+        x <- x[!grepl("[0-9]+\\.[0]+$", x$longitude), ]
+        x <- x[!grepl("[0-9]+\\.[0]+$", x$latitude), ]
+      }
+    )
+  }
+  row.names(incomp) <- NULL
+  row.names(x) <- NULL
+  structure(reassign(x), coord_imprecise = incomp)
+}
+
 #' @export
 #' @rdname coords
 coord_impossible <- function(x, lat = NULL, lon = NULL, drop = TRUE) {
   x <- do_coords(x, lat, lon)
   no_nas <- x[complete.cases(x$latitude, x$longitude), ]
-  np <- na.omit(no_nas[!abs(no_nas$latitude) <= 90 | !abs(no_nas$longitude) <= 180, ])
+  np <- na.omit(no_nas[!abs(no_nas$latitude) <= 90 |
+    !abs(no_nas$longitude) <= 180, ])
   if (NROW(np) == 0) np <- NA
   if (drop) {
     # x <- x[abs(x$latitude) <= 90 | abs(x$longitude) <= 180, ]
@@ -148,37 +252,37 @@ coord_within <- function(x, field = NULL, country = NULL,
     pkgenv <- new.env()
     data("countriesLow", package = "rworldmap", envir = pkgenv)
   }
-  check4pkg("sp")
+  check4pkg("sf")
 
   x <- do_coords(x, lat, lon)
   if (!is.null(field)) {
-    if (!field %in% names(x)) stop("field not in input data.frame", call. = FALSE)
+    if (!field %in% names(x)) stop("field not in input data.frame",
+      call. = FALSE)
   }
 
-  class(x) <- "data.frame"
-  sp::coordinates(x) <- ~longitude + latitude
-  sp::proj4string(x) <- sp::CRS("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")
+  z <- sf::st_as_sf(x, coords = c("longitude", "latitude"))
+  z <- sf::st_set_crs(z, 4326)
   refctrys <- as.character(get("countriesLow", envir = pkgenv)@data$SOVEREIGNT)
 
   if (is.null(field)) {
     ctry <- get("countriesLow", envir = pkgenv)[refctrys == country, ]
-    bb <- suppressMessages(sp::over(x, ctry))
-    wth <- x[is.na(bb[, 1]), ]@data
   } else {
     uniqctrys <- na.omit(unique(x[field][[1]]))
     if (!all(uniqctrys %in% unique(refctrys))) {
       notmatch <- uniqctrys[!uniqctrys %in% unique(refctrys)]
-      warning("Some countries do not match reference country dataset:\n ", notmatch)
+      warning("Some countries do not match reference country dataset:\n ",
+        notmatch)
     }
     ctry <- get("countriesLow", envir = pkgenv)[refctrys %in% uniqctrys, ]
-    bb <- suppressMessages(sp::over(x, ctry))
-    wth <- x[is.na(bb[, 1]), ]@data
   }
 
+  ctry <- as(ctry, "sf")
+  bb <- sf::st_join(z, ctry, join = sf::st_within)
+  wth <- tibble::as_tibble(x[is.na(bb$ADMIN), ])
+
   if (drop) {
-    x <- x[!is.na(bb[, 1]), ]
+    x <- tibble::as_tibble(x[!is.na(bb$ADMIN), ])
   }
-  x <- as_data_frame(x@data)
   if (NROW(wth) == 0) wth <- NA
   row.names(wth) <- NULL
   row.names(x) <- NULL
@@ -225,3 +329,28 @@ coord_pol_centroids <- function(x, lat = NULL, lon = NULL, drop = TRUE) {
 #   coordinates(wc) <- ~long + lat
 #   wc
 # }
+
+#' @export
+#' @rdname coords
+coord_uncertain <- function(x, coorduncertainityLimit = 30000, drop = TRUE,
+  ignore.na = FALSE){
+
+  if(!("coordinateuncertaintyinmeters"  %in%  tolower(names(x)))){
+    stop(" 'coordinateuncertainityInMeters' variable is missing",
+      call. = FALSE)
+  }
+  names(x)[grep("coordinateuncertaintyinmeters",
+    tolower(names(x)))] <- "coordinateuncertaintyinmeters"
+  if(ignore.na) x <- x[!is.na(x$coordinateuncertaintyinmeters), ]
+  uncertain_indices <-
+    which(x$coordinateuncertaintyinmeters > coorduncertainityLimit)
+  uncertain <- x[uncertain_indices, ]
+  if (NROW(uncertain) == 0) uncertain <- NA
+  if(drop){
+    certain_indices <- setdiff(seq_len(NROW(x)), uncertain_indices)
+    x <- x[certain_indices, ]
+  }
+  row.names(uncertain) <- NULL
+  row.names(x) <- NULL
+  structure(x, coord_uncertain=uncertain)
+}
